@@ -1,4 +1,5 @@
 //! Proof of knowledge of the signature and corresponding messages as per section 4.5 of the paper
+//! https://eprint.iacr.org/2016/663.pdf
 //! # Examples
 //!
 //! Creating proof of knowledge of signature and verifying it:
@@ -67,13 +68,14 @@ use serde::{Deserialize, Serialize};
 use serde_with::serde_as;
 pub use serialization::*;
 
-/// Stateful protocol to prove knowledge of signature. The protocol randomizes the signature and executes 2 Schnorr
-/// proof of knowledge protocols with the verifier in addition to verification of the randomized signature.
-/// It contains commitment (Schnorr step 1) and witnesses to both Schnorr protocols in `sc_comm_` and `sc_wits_`
-/// respectively. The protocol executes in 2 phases, pre-challenge (`init`) which is used to create the
-/// challenge and post-challenge (`gen_proof`). Thus, several instances of the protocol can be used
-/// together where the pre-challenge phase of all protocols is used to create a combined challenge and then
-/// that challenge is used in post-challenge phase of all protocols.
+/// The BBS+ signature proves validity of a set of messages {m_i}, i in I. This stateful protocol proves knowledge of such
+/// a signature whilst selectively disclosing only a subset of the messages, {m_i} for i in a disclosed set D. The
+/// protocol randomizes the initial BBS+ signature, then condcucts 2 Schnorr PoK protocols to prove exponent knowledge
+/// for the relations in section 4.5 of the paper (refer to top). It contains commitments (Schnorr step 1; refer to schnorr_pok)
+/// and witnesses to both Schnorr protocols in `sc_comm_` and `sc_wits_` respectively. The protocol executes in 2 phases,
+/// pre-challenge (`init`) which is used to create the challenge and post-challenge (`gen_proof`). Thus, several instances of
+/// the protocol can be used together where the pre-challenge phase of all protocols is used to create a combined challenge
+/// and then that challenge is used in post-challenge phase of all protocols.
 #[serde_as]
 #[derive(Clone, PartialEq, Eq, Debug, Serialize, Deserialize)]
 pub struct PoKOfSignatureG1Protocol<E: PairingEngine> {
@@ -87,7 +89,7 @@ pub struct PoKOfSignatureG1Protocol<E: PairingEngine> {
     pub sc_comm_1: SchnorrCommitment<E::G1Affine>,
     #[serde_as(as = "[FieldBytes; 2]")]
     sc_wits_1: [E::Fr; 2],
-    /// For proving relation `g1 + h1*m1 + h2*m2 +.... + h_i*m_i` = `d*r3 + {h_0}*{-s_prime} + h1*{-m1} + h2*{-m2} + .... + h_j*m_j` for all disclosed messages `m_i` and for all undisclosed messages `m_j`
+    /// For proving relation `g1 + \sum_{i in D}(h_i*m_i)` = `d*r3 + {h_0}*{-s'} + sum_{j notin D}(h_j*m_j)`
     pub sc_comm_2: SchnorrCommitment<E::G1Affine>,
     #[serde_as(as = "Vec<FieldBytes>")]
     sc_wits_2: Vec<E::Fr>,
@@ -110,7 +112,7 @@ pub struct PoKOfSignatureG1Proof<E: PairingEngine> {
     #[serde_as(as = "AffineGroupBytes")]
     pub T1: E::G1Affine,
     pub sc_resp_1: SchnorrResponse<E::G1Affine>,
-    /// Proof of relation `g1 + h1*m1 + h2*m2 +.... + h_i*m_i` = `d*r3 + {h_0}*{-s_prime} + h1*{-m1} + h2*{-m2} + .... + h_j*m_j` for all disclosed messages `m_i` and for all undisclosed messages `m_j`
+    /// Proof of relation `g1 + h1*m1 + h2*m2 +.... + h_i*m_i` = `d*r3 + {h_0}*{-s'} + h1*{-m1} + h2*{-m2} + .... + h_j*{-m_j}` for all disclosed messages `m_i` and for all undisclosed messages `m_j`
     #[serde_as(as = "AffineGroupBytes")]
     pub T2: E::G1Affine,
     pub sc_resp_2: SchnorrResponse<E::G1Affine>,
@@ -121,14 +123,14 @@ where
     E: PairingEngine,
 {
     /// Initiate the protocol, i.e. pre-challenge phase. This will generate the randomized signature and execute
-    /// the commit-to-randomness step (step 1) of both Schnorr protocols. Accepts the indices of the
+    /// the commit-to-randomness step (Step 1) of both Schnorr protocols. Accepts the indices of the
     /// multi-message which are revealed to the verifier and thus their knowledge is not proven.
     /// Accepts blindings (randomness) to be used for any messages in the multi-message. This is useful
-    /// when some messages need to be proven same as they will generate same response (step 3 in Schnorr protocol).
-    /// If extra blindings are passed, or passed for revealed messages, they are ignored. eg. If the
-    /// multi-message is `[m_0, m_1, m_2, m_3, m_4, m_5]` and the user is providing blindings for messages
-    /// `m_0` and `m_2` and revealing messages `m_3`, `m_4` and `m_5`, `blindings` is `(0 -> m_0), (2 -> m_2)`
-    /// and `revealed_msg_indices` is `(3 -> m_3), (4 -> m_4), (5 -> m_5)`
+    /// when some messages need to be proven to be the same as they will generate the same response (step 3 in
+    /// Schnorr protocol). If extra blindings are passed, or passed for revealed messages, they are ignored.
+    /// eg. If the multi-message is `[m_0, m_1, m_2, m_3, m_4]` and the user provides blindings for messages
+    /// `m_0` and `m_2` and revealing messages `m_1`, `m_3` and `m_4`, `blindings` is `(0 -> m_0), (2 -> m_2)`
+    /// and `revealed_msg_indices` is `(1 -> m_1), (3 -> m_3), (4 -> m_4)`
     pub fn init<R: RngCore>(
         rng: &mut R,
         signature: &SignatureG1<E>,
@@ -137,17 +139,22 @@ where
         mut blindings: BTreeMap<usize, E::Fr>,
         revealed_msg_indices: BTreeSet<usize>,
     ) -> Result<Self, BBSPlusError> {
-        if messages.len() != params.max_message_count() {
-            return Err(BBSPlusError::MessageCountIncompatibleWithSigParams);
+        if messages.len() != params.supported_message_count() {
+            return Err(BBSPlusError::MessageCountIncompatibleWithSigParams(
+                messages.len(),
+                params.supported_message_count(),
+            ));
         }
 
+        // No message index should be >= max messages
         for idx in &revealed_msg_indices {
             if *idx >= messages.len() {
-                return Err(BBSPlusError::InvalidMessageIdx);
+                return Err(BBSPlusError::InvalidMessageIdx(*idx));
             }
         }
 
-        // Generate any blindings that are not explicitly passed
+        // Generate any blindings that are not explicitly passed. At the end of the loop, we should have
+        // a blinding for every message whose knowledge is to be proven
         for i in 0..messages.len() {
             if !revealed_msg_indices.contains(&i) && !blindings.contains_key(&i) {
                 blindings.insert(i, E::Fr::rand(rng));
@@ -158,6 +165,7 @@ where
         let r2 = E::Fr::rand(rng);
         let r3 = r1.inverse().unwrap();
 
+        // b = (e+x) * A = g1 + h_0*s + sum(h_i*m_i) for all i in I
         let b = params.b(
             messages
                 .iter()
@@ -165,6 +173,7 @@ where
                 .collect::<BTreeMap<usize, &E::Fr>>(),
             &signature.s,
         )?;
+
         // A' = A * r1
         let A_prime = signature.A.mul(r1.into_repr());
         let A_prime_affine = A_prime.into_affine();
@@ -178,39 +187,54 @@ where
         // s' = s - r2*r3
         let s_prime = signature.s - (r2 * r3);
 
-        // For proving relation A_bar - d == A'*{-e} + h_0*r2
+        // Following is the 1st step of the Schnorr protocol for the relation pi in the paper. pi is a
+        // conjunction of 2 relations:
+        // 1. `A_bar - d == A'*{-e} + h_0*r2`
+        // 2. `g1 + \sum_{i \in D}(h_i*m_i)` = `d*r3 + {h_0}*{-s'} + \sum_{j \notin D}(h_j*{-m_j})`
+        // for all disclosed messages `m_i` and for all undisclosed messages `m_j`.
+        // For each of the above relations, a Schnorr protocol is executed; the first to prove knowledge
+        // of `(e, r2)`, and the second of `(r3, s', {m_j}_{j \notin D})`. The secret knowledge items are
+        // referred to as witnesses, and the public items as instances.
         let bases_1 = [A_prime_affine, params.h_0.clone()];
+        let randomness_1 = vec![E::Fr::rand(rng), E::Fr::rand(rng)];
         let wits_1 = [-signature.e, r2];
-        let sc_comm_1 = SchnorrCommitment::new(&bases_1, vec![E::Fr::rand(rng), E::Fr::rand(rng)]);
 
-        // For proving relation `g1 + h1*m1 + h2*m2 +.... + h_i*m_i` = `d*r3 + {h_0}*{-s_prime} + h1*{-m1} + h2*{-m2} + .... + h_j*m_j` for all disclosed messages `m_i` and for all undisclosed messages `m_j`
-        // Usually the number of disclosed messages is much less than the number of undisclosed messages, its better to avoid negations in hidden messages and do
-        // them in revealed messages. So transform the relation
-        // `g1 + h1*m1 + h2*m2 +.... + h_i*m_i` = `d*r3 + {h_0}*{-s_prime} + h1*{-m1} + h2*{-m2} + .... + h_j*m_j` for all disclosed messages `m_i` and for all undisclosed messages `m_j`
-        // into
-        // d*{-r3} + h_0*s_prime + h1*m1 + h2*m2.... + h_j*m_j = -g1 + h1*{-m1} + h2*{-m2}.... + h_i*{-m_i} for all disclosed messages `m_i` and for all undisclosed messages `m_j`.
-        // Moreover -g1 + h1*{-m1} + h2*{-m2}.... + h_i*{-m_i} is public and can be efficiently computed as -(g1 + h1*m1 + h2*m2 +.... * h_i*m_i)
+        // Commit to randomness with `h_0` and `A'`, i.e. `bases_1[0]*randomness_1[0] + bases_1[1]*randomness_1[1]`
+        let sc_comm_1 = SchnorrCommitment::new(&bases_1, randomness_1);
+
+        // Pull the bases out of the array as array is no longer needed
+        let [A_prime_affine, h_0] = bases_1;
+
+        // For proving relation `g1 + \sum_{i \in D}(h_i*m_i)` = `d*r3 + {h_0}*{-s_prime} + \sum_{j \notin D}(h_j*{-m_j})`
+        // for all disclosed messages `m_i` and for all undisclosed messages `m_j`, usually the number of disclosed
+        // messages is much less than the number of undisclosed messages; so it is better to avoid negations in
+        // undisclosed messages and do them in disclosed messaged. So negate both sides of the relation to get:
+        // `d*{-r3} + h_0*s_prime + \sum_{j \notin D}(h_j*m_j)` = `-g1 + \sum_{i \in D}(h_i*{-m_i})`
+        // Moreover `-g1 + \sum_{i \in D}(h_i*{-m_i})` is public and can be efficiently computed as -(g1 + \sum_{i \in D}(h_i*{m_i}))
+        // Knowledge of all unrevealed messages `m_j` need to be proven in addition to knowledge of `-r3` and `s'`. Thus
+        // all `m_j`, `-r3` and `s'` are the witnesses, while all `h_j`, `d`, `h_0` and `-g1 + \sum_{i \in D}(h_i*{-m_i})` is the instance.
 
         let mut bases_2 = Vec::with_capacity(2 + blindings.len());
-        let mut scalars_2 = Vec::with_capacity(2 + blindings.len());
+        let mut randomness_2 = Vec::with_capacity(2 + blindings.len());
         let mut wits_2 = Vec::with_capacity(2 + blindings.len());
-        let [A_prime_affine, h_0] = bases_1;
         bases_2.push(d_affine);
-        scalars_2.push(E::Fr::rand(rng));
+        randomness_2.push(E::Fr::rand(rng));
         wits_2.push(-r3);
         bases_2.push(h_0);
-        scalars_2.push(E::Fr::rand(rng));
+        randomness_2.push(E::Fr::rand(rng));
         wits_2.push(s_prime);
 
+        // Capture all unrevealed messages `m_j` and corresponding `h_j`
         for i in 0..messages.len() {
             if !revealed_msg_indices.contains(&i) {
                 bases_2.push(params.h[i].clone());
-                scalars_2.push(blindings.remove(&i).unwrap());
+                randomness_2.push(blindings.remove(&i).unwrap());
                 wits_2.push(messages[i].clone());
             }
         }
 
-        let sc_comm_2 = SchnorrCommitment::new(&bases_2, scalars_2);
+        // Commit to randomness, i.e. `bases_2[0]*randomness_2[0] + bases_2[1]*randomness_2[1] + .... bases_2[j]*randomness_2[j]`
+        let sc_comm_2 = SchnorrCommitment::new(&bases_2, randomness_2);
         Ok(Self {
             A_prime: A_prime_affine,
             A_bar: A_bar.into_affine(),
@@ -222,7 +246,7 @@ where
         })
     }
 
-    /// Get the contribution of this protocol towards the challenge.
+    /// Get the contribution of this protocol towards the challenge, i.e. bytecode of items that will be hashed
     pub fn challenge_contribution<W: Write>(
         &self,
         revealed_msgs: &BTreeMap<usize, E::Fr>,
@@ -241,9 +265,11 @@ where
         )
     }
 
-    /// Generate proof. post-challenge phase of the protocol.
+    /// Generate proof. Post-challenge phase of the protocol.
     pub fn gen_proof(self, challenge: &E::Fr) -> Result<PoKOfSignatureG1Proof<E>, BBSPlusError> {
+        // Schnorr response for relation `A_bar - d == A'*{-e} + h_0*r2`
         let resp_1 = self.sc_comm_1.response(&self.sc_wits_1, challenge)?;
+        // Schnorr response for relation `g1 + \sum_{i in D}(h_i*m_i)` = `d*r3 + {h_0}*{-s'} + \sum_{j not in D}(h_j*{-m_j})`
         let resp_2 = self.sc_comm_2.response(&self.sc_wits_2, challenge)?;
 
         Ok(PoKOfSignatureG1Proof {
@@ -283,24 +309,25 @@ where
         T1.serialize_unchecked(&mut writer)?;
 
         // For 2nd Schnorr
-        // `bases_disclosed` and `exponents` below are used to create g1 * h1^-m1 * h2^-m2.... for all disclosed messages m_i
-        let mut bases_disclosed = Vec::with_capacity(1 + revealed_msgs.len());
+        // `bases_revealed` and `exponents` below are used to create g1 + \sum_{i in D}(h_i*m_i)
+        // Note: exponents are really scalars here
+        let mut bases_revealed = Vec::with_capacity(1 + revealed_msgs.len());
         let mut exponents = Vec::with_capacity(1 + revealed_msgs.len());
 
         params.g1.serialize_unchecked(&mut writer)?;
-        bases_disclosed.push(params.g1);
+        bases_revealed.push(params.g1);
         let r = E::Fr::one().into_repr();
         r.serialize_unchecked(&mut writer)?;
         exponents.push(r);
         for (i, msg) in revealed_msgs {
             assert!(*i < params.h.len());
             params.h[*i].serialize_unchecked(&mut writer)?;
-            bases_disclosed.push(params.h[*i]);
+            bases_revealed.push(params.h[*i]);
             let r = msg.into_repr();
             r.serialize_unchecked(&mut writer)?;
             exponents.push(r);
         }
-        VariableBaseMSM::multi_scalar_mul(&bases_disclosed, &exponents)
+        VariableBaseMSM::multi_scalar_mul(&bases_revealed, &exponents)
             .serialize_unchecked(&mut writer)?;
         T2.serialize_unchecked(&mut writer).map_err(|e| e.into())
     }
@@ -310,7 +337,7 @@ impl<E> PoKOfSignatureG1Proof<E>
 where
     E: PairingEngine,
 {
-    /// Verify is the proof is valid
+    /// Verify if the proof is valid
     pub fn verify(
         &self,
         revealed_msgs: &BTreeMap<usize, E::Fr>,
@@ -353,26 +380,26 @@ where
         }
 
         // Verify the 2nd Schnorr proof
-        let mut bases_2 = Vec::with_capacity(2 + params.max_message_count() - revealed_msgs.len());
+        let mut bases_2 =
+            Vec::with_capacity(2 + params.supported_message_count() - revealed_msgs.len());
         bases_2.push(self.d);
         bases_2.push(params.h_0);
 
-        let mut bases_disclosed = Vec::with_capacity(1 + revealed_msgs.len());
+        let mut bases_revealed = Vec::with_capacity(1 + revealed_msgs.len());
         let mut exponents = Vec::with_capacity(1 + revealed_msgs.len());
-        // XXX: g1 should come from a setup param and not generator
-        bases_disclosed.push(params.g1);
+        bases_revealed.push(params.g1);
         exponents.push(E::Fr::one().into_repr());
-        for i in 0..params.max_message_count() {
+        for i in 0..params.supported_message_count() {
             if revealed_msgs.contains_key(&i) {
                 let message = revealed_msgs.get(&i).unwrap();
-                bases_disclosed.push(params.h[i]);
+                bases_revealed.push(params.h[i]);
                 exponents.push(message.into_repr());
             } else {
                 bases_2.push(params.h[i]);
             }
         }
-        // pr = -g1 + h1*-m1 + h2*-m2.... = -(g1 + h1*m1 * h2*m2....) for all disclosed messages m_i
-        let pr = -VariableBaseMSM::multi_scalar_mul(&bases_disclosed, &exponents);
+        // pr = -g1 + \sum_{i in D}(h_i*{-m_i}) = -(g1 + \sum_{i in D}(h_i*{m_i}))
+        let pr = -VariableBaseMSM::multi_scalar_mul(&bases_revealed, &exponents);
         let pr = pr.into_affine();
         match self.sc_resp_2.is_valid(&bases_2, &pr, &self.T2, challenge) {
             Ok(()) => (),
@@ -422,7 +449,7 @@ where
                 adjusted_idx -= 1;
             }
         }
-        // 2 added to the index, since 0th and 1st index are reserved for `s_prime` and `r2`
+        // 2 added to the index, since 0th and 1st index are reserved for `s'` and `r2`
         let r = self.sc_resp_2.get_response(2 + adjusted_idx)?;
         Ok(r)
     }
