@@ -130,7 +130,7 @@ macro_rules! impl_sig_params {
         impl<E: PairingEngine> $name<E> {
             /// Generate params by hashing a known string. The hash function is vulnerable to timing
             /// attack but since all this is public knowledge, it is fine.
-            /// This is useful if people need to be convinced that the discrete of group elements wrt each other is not known.
+            /// This is useful if people need to be convinced that the discrete log of group elements wrt each other is not known.
             pub fn new<D: Digest>(label: &[u8], n: usize) -> Self {
                 // Need n+2 elements of signature group and 1 element of other group
                 let mut sig_group_elems = Vec::with_capacity(n + 2);
@@ -185,7 +185,7 @@ macro_rules! impl_sig_params {
                 }
             }
 
-            /// Check if no group element is zero
+            /// Check that all group elements are non-zero (returns false if any element is zero)
             pub fn is_valid(&self) -> bool {
                 !(self.g1.is_zero()
                     || self.g2.is_zero()
@@ -193,13 +193,14 @@ macro_rules! impl_sig_params {
                     || iter!(self.h).any(|v| v.is_zero()))
             }
 
-            /// Maximum supported messages in the multi-message
-            pub fn max_message_count(&self) -> usize {
+            /// Number of messages supported in the multi-message
+            pub fn supported_message_count(&self) -> usize {
                 self.h.len()
             }
 
             /// Commit to given messages using the parameters and the given blinding as a Pedersen commitment.
-            /// Eg. if given messages `m_i`, `m_j`, and `m_k` in the map, the commitment is
+            /// Eg. if given messages `m_i`, `m_j`, and `m_k` in the map, the commitment converts messages to
+            /// scalars and multiplies them by the parameter curve points:
             /// `params.h_0 * blinding + params.h_i * m_i + params.h_j * m_j + params.h_k * m_k`
             /// Computes using multi-scalar multiplication
             pub fn commit_to_messages(
@@ -212,9 +213,12 @@ macro_rules! impl_sig_params {
                     Vec<E::$group_affine>,
                     Vec<<<E as ark_ec::PairingEngine>::Fr as PrimeField>::BigInt>,
                 ) = {
+                    // Need to manually check that no message index exceeds the maximum number of messages allowed
+                    // because this function can be called with only uncommitted messages; so size of BTreeMap is
+                    // not representatitve of the number of messages
                     for (i, _) in messages.iter() {
-                        if *i >= self.max_message_count() {
-                            return Err(BBSPlusError::InvalidMessageIdx);
+                        if *i >= self.supported_message_count() {
+                            return Err(BBSPlusError::InvalidMessageIdx(*i));
                         }
                     }
                     into_iter!(messages)
@@ -227,8 +231,9 @@ macro_rules! impl_sig_params {
                     let mut bases = Vec::with_capacity(messages.len());
                     let mut scalars = Vec::with_capacity(messages.len());
                     for (i, msg) in messages.into_iter() {
-                        if i >= self.max_message_count() {
-                            return Err(BBSPlusError::InvalidMessageIdx);
+                        // No message index should be >= max supported messages
+                        if i >= self.supported_message_count() {
+                            return Err(BBSPlusError::InvalidMessageIdx(i));
                         }
                         bases.push(self.h[i].clone());
                         scalars.push(msg.into_repr());
@@ -241,7 +246,8 @@ macro_rules! impl_sig_params {
                 Ok(VariableBaseMSM::multi_scalar_mul(&bases, &scalars).into_affine())
             }
 
-            /// Compute `b` from the paper. Commits to the given messages and adds `self.g1` to it
+            /// Compute `b` from the paper (equivalently 'A*{e+x}').
+            /// Commits to the given messages and adds `self.g1` to it,
             /// `b = g_1 + h_0 * s + sum(h_i * m_i)` for all indices `i` in the map.
             pub fn b(
                 &self,
@@ -258,8 +264,8 @@ macro_rules! impl_sig_params {
 macro_rules! impl_public_key {
     ( $name:ident, $group:ident, $params:ident ) => {
         /// Public key of the signer. The signer can use the same public key with different
-        /// signature parameters provided all parameters use same `g2` to sign different sized
-        /// multi-messages. This helps the signer minimize his secret key storage.
+        /// signature parameters to sign different multi-messages, provided that parameter
+        /// `g2` is consistent with the 'g2' used to generate the public key.
         #[serde_as]
         #[derive(
             Clone,
@@ -279,7 +285,7 @@ macro_rules! impl_public_key {
         where
             E: PairingEngine,
         {
-            /// Generate public key from given secret key and signature parameters
+            /// Generate public key from given secret key and signature parameter g_2
             pub fn generate_using_secret_key(
                 secret_key: &SecretKey<E::Fr>,
                 params: &$params<E>,
@@ -367,6 +373,9 @@ mod tests {
     use ark_bls12_381::Bls12_381;
     use ark_std::rand::{rngs::StdRng, SeedableRng};
     use blake2::Blake2b;
+    use schnorr_pok::compute_random_oracle_challenge;
+
+    type Fr = <Bls12_381 as PairingEngine>::Fr;
 
     macro_rules! test_serz_des {
         ($keypair:ident, $public_key:ident, $params:ident, $rng:ident, $message_count: ident) => {
@@ -456,4 +465,70 @@ mod tests {
         test_params!(SignatureParamsG1, message_count);
         test_params!(SignatureParamsG2, message_count);
     }
+
+    #[test]
+    fn proof_of_knowledge_of_public_key() {
+        macro_rules! check {
+            ($group_affine:ident, $protocol_name:ident, $proof_name:ident, $public_key:ident, $params:ident) => {
+                let mut rng = StdRng::seed_from_u64(0u64);
+                let params = $params::<Bls12_381>::new::<Blake2b>("test".as_bytes(), 5);
+                let seed = [0, 1, 2, 10, 11];
+                let sk = SecretKey::generate_using_seed::<Blake2b>(&seed);
+                let pk = $public_key::<Bls12_381>::generate_using_secret_key(&sk, &params);
+
+                let base = &params.g2;
+                let witness = sk.0.clone();
+                let blinding = Fr::rand(&mut rng);
+
+                let protocol = $protocol_name::<<Bls12_381 as PairingEngine>::$group_affine>::init(
+                    witness, blinding, base,
+                );
+
+                let mut chal_contrib_prover = vec![];
+                protocol
+                    .challenge_contribution(base, &pk.0, &mut chal_contrib_prover)
+                    .unwrap();
+
+                test_serialization!(
+                    $protocol_name<<Bls12_381 as PairingEngine>::$group_affine>,
+                    protocol
+                );
+
+                let challenge_prover =
+                    compute_random_oracle_challenge::<Fr, Blake2b>(&chal_contrib_prover);
+                let proof = protocol.gen_proof(&challenge_prover);
+
+                let mut chal_contrib_verifier = vec![];
+                proof
+                    .challenge_contribution(base, &pk.0, &mut chal_contrib_verifier)
+                    .unwrap();
+
+                let challenge_verifier =
+                    compute_random_oracle_challenge::<Fr, Blake2b>(&chal_contrib_verifier);
+                assert!(proof.verify(&pk.0, base, &challenge_verifier));
+                assert_eq!(chal_contrib_prover, chal_contrib_verifier);
+                assert_eq!(challenge_prover, challenge_verifier);
+
+                test_serialization!(
+                    $proof_name<<Bls12_381 as PairingEngine>::$group_affine>,
+                    proof
+                );
+            };
+        }
+
+        check!(
+            G2Affine,
+            PoKSecretKeyInPublicKeyG2,
+            PoKSecretKeyInPublicKeyG2Proof,
+            PublicKeyG2,
+            SignatureParamsG1
+        );
+        check!(
+            G1Affine,
+            PoKSecretKeyInPublicKeyG1,
+            PoKSecretKeyInPublicKeyG1Proof,
+            PublicKeyG1,
+            SignatureParamsG2
+        );
+    }
 }
