@@ -154,8 +154,11 @@ macro_rules! impl_signature_alg {
                 if messages.is_empty() {
                     return Err(BBSPlusError::NoMessageToSign);
                 }
-                if messages.len() != params.max_message_count() {
-                    return Err(BBSPlusError::MessageCountIncompatibleWithSigParams);
+                if messages.len() != params.supported_message_count() {
+                    return Err(BBSPlusError::MessageCountIncompatibleWithSigParams(
+                        messages.len(),
+                        params.supported_message_count(),
+                    ));
                 }
                 // Create map of msg index (0-based) -> message
                 let msg_map: BTreeMap<usize, &E::Fr> =
@@ -189,12 +192,16 @@ macro_rules! impl_signature_alg {
                 }
                 // `>` as commitment will have 0 or more messages. In practice, commitment should have
                 // at least 1 message
-                if uncommitted_messages.len() > params.max_message_count() {
-                    return Err(BBSPlusError::MessageCountIncompatibleWithSigParams);
+                if uncommitted_messages.len() > params.supported_message_count() {
+                    return Err(BBSPlusError::MessageCountIncompatibleWithSigParams(
+                        uncommitted_messages.len(),
+                        params.supported_message_count(),
+                    ));
                 }
 
                 let s = E::Fr::rand(rng);
-                // `b` is the part of signature on uncommitted messages, i.e. partial_sig = g_1 + {h_0}*s + sum(h_i * m_i) for all i in uncommitted_messages
+                // `b` is the part of signature on uncommitted messages,
+                // i.e. partial_sig = g_1 + {h_0}*s + sum(h_i * m_i) for all i in uncommitted_messages
                 let b = params.b(uncommitted_messages, &s)?;
 
                 let e = E::Fr::rand(rng);
@@ -235,8 +242,11 @@ macro_rules! impl_signature_alg {
                 if messages.is_empty() {
                     return Err(BBSPlusError::NoMessageToSign);
                 }
-                if messages.len() != params.max_message_count() {
-                    return Err(BBSPlusError::MessageCountIncompatibleWithSigParams);
+                if messages.len() != params.supported_message_count() {
+                    return Err(BBSPlusError::MessageCountIncompatibleWithSigParams(
+                        messages.len(),
+                        params.supported_message_count(),
+                    ));
                 }
                 if !self.is_non_zero() {
                     return Err(BBSPlusError::ZeroSignature);
